# ANNEX 3: OSM Information Model

## YANG model in OSM repos

YANG models can be found in the IM repo under the models folder: <https://osm.etsi.org/gitweb/?p=osm/IM.git;a=tree>

OSM uses [pyang](https://github.com/mbj4668/pyang) and [pyangbind](https://github.com/robshakir/pyangbind) to generate Python classes used by the different OSM components.

Since Release NINE, OSM IM is fully aligned with [ETSI NFV SOL006](https://www.etsi.org/deliver/etsi_gs/NFV-SOL/001_099/006/02.06.01_60/gs_NFV-SOL006v020601p.pdf). OSM Information Model augments [ETSI NFV SOL006 YANG models](https://forge.etsi.org/rep/nfv/SOL006/tree/v2.6.1) by adding the support of Network Slices, day-1 and day-2 primitives at VNF and NS level, and Enhanced Platform Awareness for dataplane workloads. Augments can be found in this folder: <https://osm.etsi.org/gitweb/?p=osm/IM.git;a=tree;f=models/augments>

## OSM IM tree representation

Below you can find tree representations of the VNFD (VNF Descriptor), NSD (Network Service Descriptor), NST (Network Slice Template), VNFR (VNF Record), NSR (Network Service Record), NSI (Network Slice Instance), both in navigable and plain text formats.

### VNFD tree

**Navigable version:**

**<http://osm-download.etsi.org/repository/osm/debian/ReleaseNINE/docs/osm-im/osm_im_trees/etsi-nfv-vnfd.html>**

**Plain text version:**

```text
module: etsi-nfv-vnfd
  +--rw vnfd!
     +--rw id?                              string
     +--rw provider                         string
     +--rw product-name                     string
     +--rw software-version                 string
     +--rw version                          string
     +--rw product-info-name?               string
     +--rw product-info-description?        string
     +--rw vnfm-info*                       string
     +--rw localization-language?           string
     +--rw default-localization-language?   string
     +--rw vdu* [id]
     |  +--rw id                         string
     |  +--rw name                       string
     |  +--rw description?               string
     |  +--rw int-cpd* [id]
     |  |  +--rw int-virtual-link-desc?                   -> ../../../int-virtual-link-desc/id
     |  |  +--rw bitrate-requirement?                     uint64
     |  |  +--rw virtual-network-interface-requirement* [name]
     |  |  |  +--rw name                              string
     |  |  |  +--rw description?                      string
     |  |  |  +--rw support-mandatory?                boolean
     |  |  |  +--rw network-interface-requirements* [key]
     |  |  |     +--rw key      string
     |  |  |     +--rw value?   string
     |  |  +--rw nicio-requirements?                      -> ../../../virtual-compute-desc/id
     |  |  +--rw order*                                   uint32
     |  |  +--rw id                                       string
     |  |  +--rw layer-protocol*                          identityref
     |  |  +--rw role?                                    identityref
     |  |  +--rw description?                             string
     |  |  +--rw protocol* [associated-layer-protocol]
     |  |  |  +--rw associated-layer-protocol    identityref
     |  |  |  +--rw address-data* [type]
     |  |  |     +--rw type               identityref
     |  |  |     +--rw l2-address-data
     |  |  |     |  +--rw mac-address-assignment    boolean
     |  |  |     +--rw l3-address-data
     |  |  |        +--rw ip-address-assignment     boolean
     |  |  |        +--rw floating-ip-activated     boolean
     |  |  |        +--rw ip-address-type?          enumeration
     |  |  |        +--rw number-of-ip-addresses?   uint32
     |  |  +--rw trunk-mode?                              boolean
     |  |  +--rw security-group-rule-id?                  -> ../../../security-group-rule/id
     |  +--rw virtual-compute-desc?      -> ../../virtual-compute-desc/id
     |  +--rw virtual-storage-desc*      -> ../../virtual-storage-desc/id
     |  +--rw boot-order* [key]
     |  |  +--rw key      uint32
     |  |  +--rw value?   -> ../../virtual-storage-desc
     |  +--rw sw-image-desc?             -> ../../sw-image-desc/id
     |  +--rw nfvi-constraint*           string
     |  +--rw monitoring-parameter* [id]
     |  |  +--rw id                    string
     |  |  +--rw name?                 string
     |  |  +--rw performance-metric?   string
     |  |  +--rw collection-period?    uint64
     |  +--rw configurable-properties* [key]
     |  |  +--rw key      string
     |  |  +--rw value?   string
     |  +--rw boot-data?                 string
     +--rw virtual-compute-desc* [id]
     |  +--rw id                               string
     |  +--rw logical-node* [id]
     |  |  +--rw id                    string
     |  |  +--rw requirement-detail* [key]
     |  |     +--rw key      string
     |  |     +--rw value?   string
     |  +--rw request-additional-capability* [name]
     |  |  +--rw name                             string
     |  |  +--rw support-mandatory?               boolean
     |  |  +--rw min-version?                     string
     |  |  +--rw preferred-version?               string
     |  |  +--rw target-performance-parameters* [key]
     |  |     +--rw key      string
     |  |     +--rw value?   string
     |  +--rw compute-requirements?            string
     |  +--rw virtual-memory
     |  |  +--rw size?                       decimal64
     |  |  +--rw over-subscription-policy?   string
     |  |  +--rw vdu-mem-requirements* [key]
     |  |  |  +--rw key      string
     |  |  |  +--rw value?   string
     |  |  +--rw numa-enabled?               boolean
     |  +--rw virtual-cpu
     |     +--rw cpu-architecture?          string
     |     +--rw num-virtual-cpu?           uint16
     |     +--rw clock?                     uint32
     |     +--rw oversubscription-policy?   string
     |     +--rw vdu-cpu-requirements* [key]
     |     |  +--rw key      string
     |     |  +--rw value?   string
     |     +--rw pinning!
     |        +--rw policy?   enumeration
     |        +--rw rule* [key]
     |           +--rw key      string
     |           +--rw value?   string
     +--rw virtual-storage-desc* [id]
     |  +--rw id                          string
     |  +--rw type-of-storage?            identityref
     |  +--rw size-of-storage?            uint64
     |  +--rw vdu-storage-requirements* [key]
     |  |  +--rw key      string
     |  |  +--rw value?   string
     |  +--rw rdma-enabled?               boolean
     |  +--rw sw-image-desc?              -> ../../sw-image-desc/id
     +--rw sw-image-desc* [id]
     |  +--rw id                                      string
     |  +--rw name                                    string
     |  +--rw version                                 string
     |  +--rw checksum
     |  |  +--rw algorithm    identityref
     |  |  +--rw hash         string
     |  +--rw container-format?                       enumeration
     |  +--rw disk-format?                            enumeration
     |  +--rw min-disk                                uint64
     |  +--rw min-ram?                                decimal64
     |  +--rw size                                    uint64
     |  +--rw image?                                  inet:uri
     |  +--rw operating-system?                       string
     |  +--rw supported-virtualization-environment*   string
     +--rw int-virtual-link-desc* [id]
     |  +--rw id                       string
     |  +--rw flavour* [id]
     |  |  +--rw id     string
     |  |  +--rw qos!
     |  |     +--rw latency                   uint32
     |  |     +--rw packet-delay-variation    uint32
     |  |     +--rw packet-loss-ratio?        decimal64
     |  +--rw connectivity-type
     |  |  +--rw layer-protocol*   identityref
     |  |  +--rw flow-pattern?     flow-pattern
     |  +--rw test-access*             string
     |  +--rw description?             string
     |  +--rw monitoring-parameters* [id]
     |     +--rw id                    string
     |     +--rw name?                 string
     |     +--rw performance-metric?   string
     |     +--rw collection-period?    uint64
     +--rw security-group-rule* [id]
     |  +--rw id                string
     |  +--rw description?      string
     |  +--rw direction?        enumeration
     |  +--rw ether-type?       enumeration
     |  +--rw protocol?         enumeration
     |  +--rw port-range-min?   uint16
     |  +--rw port-range-max?   uint16
     +--rw ext-cpd* [id]
     |  +--rw (cp-connection)?
     |  |  +--:(int-virtual-link-desc)
     |  |  |  +--rw int-virtual-link-desc?             -> ../../int-virtual-link-desc/id
     |  |  +--:(int-cpd)
     |  |     +--rw int-cpd
     |  |        +--rw vdu-id?   -> ../../../vdu/id
     |  |        +--rw cpd?      -> deref(../vdu-id)/../int-cpd/id
     |  +--rw virtual-network-interface-requirement* [name]
     |  |  +--rw name                              string
     |  |  +--rw description?                      string
     |  |  +--rw support-mandatory?                boolean
     |  |  +--rw network-interface-requirements* [key]
     |  |     +--rw key      string
     |  |     +--rw value?   string
     |  +--rw nicio-requirements?                      -> ../../virtual-compute-desc/id
     |  +--rw id                                       string
     |  +--rw layer-protocol*                          identityref
     |  +--rw role?                                    identityref
     |  +--rw description?                             string
     |  +--rw protocol* [associated-layer-protocol]
     |  |  +--rw associated-layer-protocol    identityref
     |  |  +--rw address-data* [type]
     |  |     +--rw type               identityref
     |  |     +--rw l2-address-data
     |  |     |  +--rw mac-address-assignment    boolean
     |  |     +--rw l3-address-data
     |  |        +--rw ip-address-assignment     boolean
     |  |        +--rw floating-ip-activated     boolean
     |  |        +--rw ip-address-type?          enumeration
     |  |        +--rw number-of-ip-addresses?   uint32
     |  +--rw trunk-mode?                              boolean
     |  +--rw security-group-rule-id?                  -> ../../security-group-rule/id
     +--rw df* [id]
     |  +--rw id                                 string
     |  +--rw description?                       string
     |  +--rw vdu-profile* [id]
     |  |  +--rw id                                      -> ../../../vdu/id
     |  |  +--rw min-number-of-instances?                uint16
     |  |  +--rw max-number-of-instances?                uint16
     |  |  +--rw local-affinity-or-anti-affinity-rule* [affinity-type affinity-scope]
     |  |  |  +--rw affinity-type     affinity-type
     |  |  |  +--rw affinity-scope    affinity-scope
     |  |  +--rw affinity-or-anti-affinity-group* [id]
     |  |     +--rw id    -> ../../../affinity-or-anti-affinity-group/id
     |  +--rw virtual-link-profile* [id flavour]
     |  |  +--rw id                                      -> ../../../int-virtual-link-desc/id
     |  |  +--rw flavour                                 -> deref(../../../ext-cpd/int-virtual-link-desc)/../flavour/id
     |  |  +--rw local-affinity-or-anti-affinity-rule* [affinity-type affinity-scope]
     |  |  |  +--rw affinity-type     affinity-type
     |  |  |  +--rw affinity-scope    affinity-scope
     |  |  +--rw affinity-or-anti-affinity-group* [id]
     |  |  |  +--rw id    -> ../../../affinity-or-anti-affinity-group/id
     |  |  +--rw max-bit-rate-requirements
     |  |  |  +--rw root    uint32
     |  |  |  +--rw leaf?   uint32
     |  |  +--rw min-bit-rate-requirements
     |  |  |  +--rw root    uint32
     |  |  |  +--rw leaf?   uint32
     |  |  +--rw virtual-link-protocol-data
     |  |     +--rw associated-layer-protocol?   identityref
     |  |     +--rw l2-protocol-data
     |  |     |  +--rw name?               string
     |  |     |  +--rw network-type?       enumeration
     |  |     |  +--rw vlan-transparent?   boolean
     |  |     |  +--rw mtu?                uint16
     |  |     +--rw l3-protocol-data
     |  |        +--rw name?                  string
     |  |        +--rw ip-version?            enumeration
     |  |        +--rw cidr?                  string
     |  |        +--rw ip-allocation-pools*   string
     |  |        +--rw gateway-ip?            inet:ip-address
     |  |        +--rw dhcp-enabled?          boolean
     |  |        +--rw ipv6-address-mode?     enumeration
     |  +--rw instantiation-level* [id]
     |  |  +--rw id              string
     |  |  +--rw description?    string
     |  |  +--rw vdu-level* [vdu-id]
     |  |  |  +--rw vdu-id                 -> ../../../../vdu/id
     |  |  |  +--rw number-of-instances?   uint16
     |  |  +--rw scaling-info* [scaling-aspect-id]
     |  |     +--rw scaling-aspect-id    -> ../../../scaling-aspect/id
     |  |     +--rw scale-level?         uint32
     |  +--rw default-instantiation-level?       -> ../instantiation-level/id
     |  +--rw supported-operation*               identityref
     |  +--rw lcm-operations-configuration
     |  |  +--rw instantiate-vnf-op-config
     |  |  |  +--rw parameter* [key]
     |  |  |     +--rw key      string
     |  |  |     +--rw value?   string
     |  |  +--rw scale-vnf-op-config
     |  |  |  +--rw parameter* [key]
     |  |  |  |  +--rw key      string
     |  |  |  |  +--rw value?   string
     |  |  |  +--rw scaling-by-more-than-one-step-supported?   boolean
     |  |  +--rw scale-vnf-to-level-op-config
     |  |  |  +--rw parameter* [key]
     |  |  |  |  +--rw key      string
     |  |  |  |  +--rw value?   string
     |  |  |  +--rw arbitrary-target-levels-supported?   boolean
     |  |  +--rw heal-vnf-op-config
     |  |  |  +--rw parameter* [key]
     |  |  |  |  +--rw key      string
     |  |  |  |  +--rw value?   string
     |  |  |  +--rw cause*       string
     |  |  +--rw terminate-vnf-op-config
     |  |  |  +--rw min-graceful-termination?               yang:timeticks
     |  |  |  +--rw max-recommended-graceful-termination?   yang:timeticks
     |  |  |  +--rw parameter* [key]
     |  |  |     +--rw key      string
     |  |  |     +--rw value?   string
     |  |  +--rw operate-vnf-op-config
     |  |  |  +--rw min-graceful-stop-timeout?               yang:timeticks
     |  |  |  +--rw max-recommended-graceful-stop-timeout?   yang:timeticks
     |  |  |  +--rw parameter* [key]
     |  |  |     +--rw key      string
     |  |  |     +--rw value?   string
     |  |  +--rw change-vnf-flavour-op-config
     |  |  |  +--rw parameter* [key]
     |  |  |     +--rw key      string
     |  |  |     +--rw value?   string
     |  |  +--rw change-ext-vnf-connectivity-op-config
     |  |     +--rw parameter* [key]
     |  |        +--rw key      string
     |  |        +--rw value?   string
     |  +--rw affinity-or-anti-affinity-group* [id]
     |  |  +--rw id       string
     |  |  +--rw type     affinity-type
     |  |  +--rw scope    affinity-scope
     |  +--rw indicator* [id]
     |  |  +--rw id                 string
     |  |  +--rw name?              string
     |  |  +--rw indicator-value?   string
     |  |  +--rw source?            enumeration
     |  +--rw supported-vnf-interfaces* [name]
     |  |  +--rw name                 enumeration
     |  |  +--rw cpd-id*              -> ../../../ext-cpd/id
     |  |  +--rw interface-details* [key]
     |  |     +--rw key      string
     |  |     +--rw value?   string
     |  +--rw monitoring-parameter* [id]
     |  |  +--rw id                    string
     |  |  +--rw name?                 string
     |  |  +--rw performance-metric?   string
     |  |  +--rw collection-period?    uint64
     |  +--rw scaling-aspect* [id]
     |     +--rw id                      string
     |     +--rw name?                   string
     |     +--rw description?            string
     |     +--rw max-scale-level?        uint32
     |     +--rw aspect-delta-details
     |        +--rw deltas* [id]
     |        |  +--rw id                             string
     |        |  +--rw vdu-delta* [id]
     |        |  |  +--rw id                     -> ../../../../../../vdu/id
     |        |  |  +--rw number-of-instances?   uint32
     |        |  +--rw virtual-link-bit-rate-delta* [id]
     |        |     +--rw id                       string
     |        |     +--rw bit-rate-requirements
     |        |        +--rw root    uint32
     |        |        +--rw leaf?   uint32
     |        +--rw step-deltas?   -> ../deltas/id
     +--rw configurable-properties
     |  +--rw is-auto-scalable-enabled?           boolean
     |  +--rw is-auto-heal-enabled?               boolean
     |  +--rw additional-configurable-property* [key]
     |     +--rw key      string
     |     +--rw value?   string
     +--rw modifiable-attributes
     |  +--rw extension*   string
     |  +--rw metadata*    string
     +--rw lifecycle-management-script* [id]
     |  +--rw id                      string
     |  +--rw event*                  internal-lifecycle-management-script-event
     |  +--rw lcm-transition-event*   string
     |  +--rw script?                 string
     |  +--rw script-dsl?             string
     |  +--rw script-input* [key]
     |     +--rw key      string
     |     +--rw value?   string
     +--rw element-group* [id]
     |  +--rw id                   string
     |  +--rw description?         string
     |  +--rw vdu*                 -> ../../vdu/id
     |  +--rw virtual-link-desc*   -> ../../int-virtual-link-desc/id
     +--rw indicator* [id]
     |  +--rw id                 string
     |  +--rw name?              string
     |  +--rw indicator-value*   string
     |  +--rw source?            enumeration
     +--rw auto-scale*                      string
```

### NSD tree

**Navigable version:**

**<http://osm-download.etsi.org/repository/osm/debian/ReleaseNINE/docs/osm-im/osm_im_trees/etsi-nfv-nsd.html>**

**Plain text version:**

```text
module: etsi-nfv-nsd
  +--rw nsd
     +--rw vnfd* [id]
     |  +--rw id                               string
     |  +--rw provider                         string
     |  +--rw product-name                     string
     |  +--rw software-version                 string
     |  +--rw version                          string
     |  +--rw product-info-name?               string
     |  +--rw product-info-description?        string
     |  +--rw vnfm-info*                       string
     |  +--rw localization-language?           string
     |  +--rw default-localization-language?   string
     |  +--rw vdu* [id]
     |  |  +--rw id                         string
     |  |  +--rw name                       string
     |  |  +--rw description?               string
     |  |  +--rw int-cpd* [id]
     |  |  |  +--rw int-virtual-link-desc?                   -> ../../../int-virtual-link-desc/id
     |  |  |  +--rw bitrate-requirement?                     uint64
     |  |  |  +--rw virtual-network-interface-requirement* [name]
     |  |  |  |  +--rw name                              string
     |  |  |  |  +--rw description?                      string
     |  |  |  |  +--rw support-mandatory?                boolean
     |  |  |  |  +--rw network-interface-requirements* [key]
     |  |  |  |     +--rw key      string
     |  |  |  |     +--rw value?   string
     |  |  |  +--rw nicio-requirements?                      -> ../../../virtual-compute-desc/id
     |  |  |  +--rw order*                                   uint32
     |  |  |  +--rw id                                       string
     |  |  |  +--rw layer-protocol*                          identityref
     |  |  |  +--rw role?                                    identityref
     |  |  |  +--rw description?                             string
     |  |  |  +--rw protocol* [associated-layer-protocol]
     |  |  |  |  +--rw associated-layer-protocol    identityref
     |  |  |  |  +--rw address-data* [type]
     |  |  |  |     +--rw type               identityref
     |  |  |  |     +--rw l2-address-data
     |  |  |  |     |  +--rw mac-address-assignment    boolean
     |  |  |  |     +--rw l3-address-data
     |  |  |  |        +--rw ip-address-assignment     boolean
     |  |  |  |        +--rw floating-ip-activated     boolean
     |  |  |  |        +--rw ip-address-type?          enumeration
     |  |  |  |        +--rw number-of-ip-addresses?   uint32
     |  |  |  +--rw trunk-mode?                              boolean
     |  |  |  +--rw security-group-rule-id?                  -> ../../../security-group-rule/id
     |  |  +--rw virtual-compute-desc?      -> ../../virtual-compute-desc/id
     |  |  +--rw virtual-storage-desc*      -> ../../virtual-storage-desc/id
     |  |  +--rw boot-order* [key]
     |  |  |  +--rw key      uint32
     |  |  |  +--rw value?   -> ../../virtual-storage-desc
     |  |  +--rw sw-image-desc?             -> ../../sw-image-desc/id
     |  |  +--rw nfvi-constraint*           string
     |  |  +--rw monitoring-parameter* [id]
     |  |  |  +--rw id                    string
     |  |  |  +--rw name?                 string
     |  |  |  +--rw performance-metric?   string
     |  |  |  +--rw collection-period?    uint64
     |  |  +--rw configurable-properties* [key]
     |  |  |  +--rw key      string
     |  |  |  +--rw value?   string
     |  |  +--rw boot-data?                 string
     |  +--rw virtual-compute-desc* [id]
     |  |  +--rw id                               string
     |  |  +--rw logical-node* [id]
     |  |  |  +--rw id                    string
     |  |  |  +--rw requirement-detail* [key]
     |  |  |     +--rw key      string
     |  |  |     +--rw value?   string
     |  |  +--rw request-additional-capability* [name]
     |  |  |  +--rw name                             string
     |  |  |  +--rw support-mandatory?               boolean
     |  |  |  +--rw min-version?                     string
     |  |  |  +--rw preferred-version?               string
     |  |  |  +--rw target-performance-parameters* [key]
     |  |  |     +--rw key      string
     |  |  |     +--rw value?   string
     |  |  +--rw compute-requirements?            string
     |  |  +--rw virtual-memory
     |  |  |  +--rw size?                       decimal64
     |  |  |  +--rw over-subscription-policy?   string
     |  |  |  +--rw vdu-mem-requirements* [key]
     |  |  |  |  +--rw key      string
     |  |  |  |  +--rw value?   string
     |  |  |  +--rw numa-enabled?               boolean
     |  |  +--rw virtual-cpu
     |  |     +--rw cpu-architecture?          string
     |  |     +--rw num-virtual-cpu?           uint16
     |  |     +--rw clock?                     uint32
     |  |     +--rw oversubscription-policy?   string
     |  |     +--rw vdu-cpu-requirements* [key]
     |  |     |  +--rw key      string
     |  |     |  +--rw value?   string
     |  |     +--rw pinning!
     |  |        +--rw policy?   enumeration
     |  |        +--rw rule* [key]
     |  |           +--rw key      string
     |  |           +--rw value?   string
     |  +--rw virtual-storage-desc* [id]
     |  |  +--rw id                          string
     |  |  +--rw type-of-storage?            identityref
     |  |  +--rw size-of-storage?            uint64
     |  |  +--rw vdu-storage-requirements* [key]
     |  |  |  +--rw key      string
     |  |  |  +--rw value?   string
     |  |  +--rw rdma-enabled?               boolean
     |  |  +--rw sw-image-desc?              -> ../../sw-image-desc/id
     |  +--rw sw-image-desc* [id]
     |  |  +--rw id                                      string
     |  |  +--rw name                                    string
     |  |  +--rw version                                 string
     |  |  +--rw checksum
     |  |  |  +--rw algorithm    identityref
     |  |  |  +--rw hash         string
     |  |  +--rw container-format?                       enumeration
     |  |  +--rw disk-format?                            enumeration
     |  |  +--rw min-disk                                uint64
     |  |  +--rw min-ram?                                decimal64
     |  |  +--rw size                                    uint64
     |  |  +--rw image?                                  inet:uri
     |  |  +--rw operating-system?                       string
     |  |  +--rw supported-virtualization-environment*   string
     |  +--rw int-virtual-link-desc* [id]
     |  |  +--rw id                       string
     |  |  +--rw flavour* [id]
     |  |  |  +--rw id     string
     |  |  |  +--rw qos!
     |  |  |     +--rw latency                   uint32
     |  |  |     +--rw packet-delay-variation    uint32
     |  |  |     +--rw packet-loss-ratio?        decimal64
     |  |  +--rw connectivity-type
     |  |  |  +--rw layer-protocol*   identityref
     |  |  |  +--rw flow-pattern?     flow-pattern
     |  |  +--rw test-access*             string
     |  |  +--rw description?             string
     |  |  +--rw monitoring-parameters* [id]
     |  |     +--rw id                    string
     |  |     +--rw name?                 string
     |  |     +--rw performance-metric?   string
     |  |     +--rw collection-period?    uint64
     |  +--rw security-group-rule* [id]
     |  |  +--rw id                string
     |  |  +--rw description?      string
     |  |  +--rw direction?        enumeration
     |  |  +--rw ether-type?       enumeration
     |  |  +--rw protocol?         enumeration
     |  |  +--rw port-range-min?   uint16
     |  |  +--rw port-range-max?   uint16
     |  +--rw ext-cpd* [id]
     |  |  +--rw (cp-connection)?
     |  |  |  +--:(int-virtual-link-desc)
     |  |  |  |  +--rw int-virtual-link-desc?             -> ../../int-virtual-link-desc/id
     |  |  |  +--:(int-cpd)
     |  |  |     +--rw int-cpd
     |  |  |        +--rw vdu-id?   -> ../../../vdu/id
     |  |  |        +--rw cpd?      -> deref(../vdu-id)/../int-cpd/id
     |  |  +--rw virtual-network-interface-requirement* [name]
     |  |  |  +--rw name                              string
     |  |  |  +--rw description?                      string
     |  |  |  +--rw support-mandatory?                boolean
     |  |  |  +--rw network-interface-requirements* [key]
     |  |  |     +--rw key      string
     |  |  |     +--rw value?   string
     |  |  +--rw nicio-requirements?                      -> ../../virtual-compute-desc/id
     |  |  +--rw id                                       string
     |  |  +--rw layer-protocol*                          identityref
     |  |  +--rw role?                                    identityref
     |  |  +--rw description?                             string
     |  |  +--rw protocol* [associated-layer-protocol]
     |  |  |  +--rw associated-layer-protocol    identityref
     |  |  |  +--rw address-data* [type]
     |  |  |     +--rw type               identityref
     |  |  |     +--rw l2-address-data
     |  |  |     |  +--rw mac-address-assignment    boolean
     |  |  |     +--rw l3-address-data
     |  |  |        +--rw ip-address-assignment     boolean
     |  |  |        +--rw floating-ip-activated     boolean
     |  |  |        +--rw ip-address-type?          enumeration
     |  |  |        +--rw number-of-ip-addresses?   uint32
     |  |  +--rw trunk-mode?                              boolean
     |  |  +--rw security-group-rule-id?                  -> ../../security-group-rule/id
     |  +--rw df* [id]
     |  |  +--rw id                                 string
     |  |  +--rw description?                       string
     |  |  +--rw vdu-profile* [id]
     |  |  |  +--rw id                                      -> ../../../vdu/id
     |  |  |  +--rw min-number-of-instances?                uint16
     |  |  |  +--rw max-number-of-instances?                uint16
     |  |  |  +--rw local-affinity-or-anti-affinity-rule* [affinity-type affinity-scope]
     |  |  |  |  +--rw affinity-type     affinity-type
     |  |  |  |  +--rw affinity-scope    affinity-scope
     |  |  |  +--rw affinity-or-anti-affinity-group* [id]
     |  |  |     +--rw id    -> ../../../affinity-or-anti-affinity-group/id
     |  |  +--rw virtual-link-profile* [id flavour]
     |  |  |  +--rw id                                      -> ../../../int-virtual-link-desc/id
     |  |  |  +--rw flavour                                 -> deref(../../../ext-cpd/int-virtual-link-desc)/../flavour/id
     |  |  |  +--rw local-affinity-or-anti-affinity-rule* [affinity-type affinity-scope]
     |  |  |  |  +--rw affinity-type     affinity-type
     |  |  |  |  +--rw affinity-scope    affinity-scope
     |  |  |  +--rw affinity-or-anti-affinity-group* [id]
     |  |  |  |  +--rw id    -> ../../../affinity-or-anti-affinity-group/id
     |  |  |  +--rw max-bit-rate-requirements
     |  |  |  |  +--rw root    uint32
     |  |  |  |  +--rw leaf?   uint32
     |  |  |  +--rw min-bit-rate-requirements
     |  |  |  |  +--rw root    uint32
     |  |  |  |  +--rw leaf?   uint32
     |  |  |  +--rw virtual-link-protocol-data
     |  |  |     +--rw associated-layer-protocol?   identityref
     |  |  |     +--rw l2-protocol-data
     |  |  |     |  +--rw name?               string
     |  |  |     |  +--rw network-type?       enumeration
     |  |  |     |  +--rw vlan-transparent?   boolean
     |  |  |     |  +--rw mtu?                uint16
     |  |  |     +--rw l3-protocol-data
     |  |  |        +--rw name?                  string
     |  |  |        +--rw ip-version?            enumeration
     |  |  |        +--rw cidr?                  string
     |  |  |        +--rw ip-allocation-pools*   string
     |  |  |        +--rw gateway-ip?            inet:ip-address
     |  |  |        +--rw dhcp-enabled?          boolean
     |  |  |        +--rw ipv6-address-mode?     enumeration
     |  |  +--rw instantiation-level* [id]
     |  |  |  +--rw id              string
     |  |  |  +--rw description?    string
     |  |  |  +--rw vdu-level* [vdu-id]
     |  |  |  |  +--rw vdu-id                 -> ../../../../vdu/id
     |  |  |  |  +--rw number-of-instances?   uint16
     |  |  |  +--rw scaling-info* [scaling-aspect-id]
     |  |  |     +--rw scaling-aspect-id    -> ../../../scaling-aspect/id
     |  |  |     +--rw scale-level?         uint32
     |  |  +--rw default-instantiation-level?       -> ../instantiation-level/id
     |  |  +--rw supported-operation*               identityref
     |  |  +--rw lcm-operations-configuration
     |  |  |  +--rw instantiate-vnf-op-config
     |  |  |  |  +--rw parameter* [key]
     |  |  |  |     +--rw key      string
     |  |  |  |     +--rw value?   string
     |  |  |  +--rw scale-vnf-op-config
     |  |  |  |  +--rw parameter* [key]
     |  |  |  |  |  +--rw key      string
     |  |  |  |  |  +--rw value?   string
     |  |  |  |  +--rw scaling-by-more-than-one-step-supported?   boolean
     |  |  |  +--rw scale-vnf-to-level-op-config
     |  |  |  |  +--rw parameter* [key]
     |  |  |  |  |  +--rw key      string
     |  |  |  |  |  +--rw value?   string
     |  |  |  |  +--rw arbitrary-target-levels-supported?   boolean
     |  |  |  +--rw heal-vnf-op-config
     |  |  |  |  +--rw parameter* [key]
     |  |  |  |  |  +--rw key      string
     |  |  |  |  |  +--rw value?   string
     |  |  |  |  +--rw cause*       string
     |  |  |  +--rw terminate-vnf-op-config
     |  |  |  |  +--rw min-graceful-termination?               yang:timeticks
     |  |  |  |  +--rw max-recommended-graceful-termination?   yang:timeticks
     |  |  |  |  +--rw parameter* [key]
     |  |  |  |     +--rw key      string
     |  |  |  |     +--rw value?   string
     |  |  |  +--rw operate-vnf-op-config
     |  |  |  |  +--rw min-graceful-stop-timeout?               yang:timeticks
     |  |  |  |  +--rw max-recommended-graceful-stop-timeout?   yang:timeticks
     |  |  |  |  +--rw parameter* [key]
     |  |  |  |     +--rw key      string
     |  |  |  |     +--rw value?   string
     |  |  |  +--rw change-vnf-flavour-op-config
     |  |  |  |  +--rw parameter* [key]
     |  |  |  |     +--rw key      string
     |  |  |  |     +--rw value?   string
     |  |  |  +--rw change-ext-vnf-connectivity-op-config
     |  |  |     +--rw parameter* [key]
     |  |  |        +--rw key      string
     |  |  |        +--rw value?   string
     |  |  +--rw affinity-or-anti-affinity-group* [id]
     |  |  |  +--rw id       string
     |  |  |  +--rw type     affinity-type
     |  |  |  +--rw scope    affinity-scope
     |  |  +--rw indicator* [id]
     |  |  |  +--rw id                 string
     |  |  |  +--rw name?              string
     |  |  |  +--rw indicator-value?   string
     |  |  |  +--rw source?            enumeration
     |  |  +--rw supported-vnf-interfaces* [name]
     |  |  |  +--rw name                 enumeration
     |  |  |  +--rw cpd-id*              -> ../../../ext-cpd/id
     |  |  |  +--rw interface-details* [key]
     |  |  |     +--rw key      string
     |  |  |     +--rw value?   string
     |  |  +--rw monitoring-parameter* [id]
     |  |  |  +--rw id                    string
     |  |  |  +--rw name?                 string
     |  |  |  +--rw performance-metric?   string
     |  |  |  +--rw collection-period?    uint64
     |  |  +--rw scaling-aspect* [id]
     |  |     +--rw id                      string
     |  |     +--rw name?                   string
     |  |     +--rw description?            string
     |  |     +--rw max-scale-level?        uint32
     |  |     +--rw aspect-delta-details
     |  |        +--rw deltas* [id]
     |  |        |  +--rw id                             string
     |  |        |  +--rw vdu-delta* [id]
     |  |        |  |  +--rw id                     -> ../../../../../../vdu/id
     |  |        |  |  +--rw number-of-instances?   uint32
     |  |        |  +--rw virtual-link-bit-rate-delta* [id]
     |  |        |     +--rw id                       string
     |  |        |     +--rw bit-rate-requirements
     |  |        |        +--rw root    uint32
     |  |        |        +--rw leaf?   uint32
     |  |        +--rw step-deltas?   -> ../deltas/id
     |  +--rw configurable-properties
     |  |  +--rw is-auto-scalable-enabled?           boolean
     |  |  +--rw is-auto-heal-enabled?               boolean
     |  |  +--rw additional-configurable-property* [key]
     |  |     +--rw key      string
     |  |     +--rw value?   string
     |  +--rw modifiable-attributes
     |  |  +--rw extension*   string
     |  |  +--rw metadata*    string
     |  +--rw lifecycle-management-script* [id]
     |  |  +--rw id                      string
     |  |  +--rw event*                  internal-lifecycle-management-script-event
     |  |  +--rw lcm-transition-event*   string
     |  |  +--rw script?                 string
     |  |  +--rw script-dsl?             string
     |  |  +--rw script-input* [key]
     |  |     +--rw key      string
     |  |     +--rw value?   string
     |  +--rw element-group* [id]
     |  |  +--rw id                   string
     |  |  +--rw description?         string
     |  |  +--rw vdu*                 -> ../../vdu/id
     |  |  +--rw virtual-link-desc*   -> ../../int-virtual-link-desc/id
     |  +--rw indicator* [id]
     |  |  +--rw id                 string
     |  |  +--rw name?              string
     |  |  +--rw indicator-value*   string
     |  |  +--rw source?            enumeration
     |  +--rw auto-scale*                      string
     +--rw nsd* [id]
     |  +--rw id                             string
     |  +--rw designer?                      string
     |  +--rw version?                       string
     |  +--rw name?                          string
     |  +--rw invariant-id?                  string
     |  +--rw nested-nsd-id*                 -> ../../nsd/id
     |  +--rw vnfd-id*                       -> ../../vnfd/id
     |  +--rw pnfd-id*                       -> ../../pnfd/id
     |  +--rw sapd* [id]
     |  |  +--rw id                         string
     |  |  +--rw address-assignment?        boolean
     |  |  +--rw (cpd-or-virtual-link)?
     |  |     +--:(virtual-link-desc)
     |  |     |  +--rw virtual-link-desc?   -> ../../virtual-link-desc/id
     |  |     +--:(vnf)
     |  |     |  +--rw vnf
     |  |     |     +--rw vnfd-id       -> ../../../../vnfd/id
     |  |     |     +--rw ext-cpd-id    -> deref(../vnfd-id)/../ext-cpd/id
     |  |     +--:(pnf)
     |  |     |  +--rw pnf
     |  |     |     +--rw pnfd-id       -> ../../../../pnfd/id
     |  |     |     +--rw ext-cpd-id    -> deref(../pnfd-id)/../ext-cpd/id
     |  |     +--:(ns)
     |  |        +--rw ns
     |  |           +--rw nsd-id        -> ../../../../nsd/id
     |  |           +--rw ext-cpd-id    -> deref(../nsd-id)/../sapd/id
     |  +--rw virtual-link-desc* [id]
     |  |  +--rw id                   string
     |  |  +--rw connectivity-type
     |  |  |  +--rw layer-protocol*   identityref
     |  |  |  +--rw flow-pattern?     flow-pattern
     |  |  +--rw df* [id]
     |  |  |  +--rw id                            string
     |  |  |  +--rw qos!
     |  |  |  |  +--rw latency                   uint32
     |  |  |  |  +--rw packet-delay-variation    uint32
     |  |  |  |  +--rw packet-loss-ratio?        decimal64
     |  |  |  |  +--rw priority?                 uint32
     |  |  |  +--rw service-availability-level?   enumeration
     |  |  +--rw test-access?         enumeration
     |  |  +--rw description?         string
     |  |  +--rw signature?           string
     |  |  +--rw algorithm?           string
     |  |  +--rw certificate?         string
     |  +--rw vnffgd* [id]
     |  |  +--rw id                         string
     |  |  +--rw vnf-profile-id*            -> ../../df/vnf-profile/id
     |  |  +--rw pnf-profile-id*            -> ../../df/pnf-profile/id
     |  |  +--rw nested-ns-profile-id*      -> ../../df/ns-profile/id
     |  |  +--rw virtual-link-profile-id?   -> ../../df/virtual-link-profile/id
     |  |  +--rw cpd-pool* [id]
     |  |  |  +--rw id                   string
     |  |  |  +--rw (constituent-base-element-id)
     |  |  |  |  +--:(vnf-profile)
     |  |  |  |  |  +--rw vnf-profile
     |  |  |  |  |     +--rw vnf-profile-id?   -> ../../../../../nsd/df/vnf-profile/id
     |  |  |  |  +--:(pnf-profile)
     |  |  |  |  |  +--rw pnf-profile
     |  |  |  |  |     +--rw pnf-profile-id?   -> ../../../../../nsd/df/pnf-profile/id
     |  |  |  |  +--:(ns-profile)
     |  |  |  |     +--rw ns-profile
     |  |  |  |        +--rw ns-profile-id?   -> ../../../../../nsd/df/ns-profile/id
     |  |  |  +--rw (constituent-cpd-id)?
     |  |  |     +--:(vnf)
     |  |  |     |  +--rw vnf
     |  |  |     |     +--rw vnfd-id?   -> ../../../../../vnfd/id
     |  |  |     |     +--rw cpd-id?    -> deref(../vnfd-id)/../ext-cpd/id
     |  |  |     +--:(pnf)
     |  |  |     |  +--rw pnf
     |  |  |     |     +--rw pnfd-id?      -> ../../../../../pnfd/id
     |  |  |     |     +--rw pnf-cpd-id?   -> deref(../pnfd-id)/../ext-cpd/id
     |  |  |     +--:(ns)
     |  |  |        +--rw ns
     |  |  |           +--rw nsd-id?       -> ../../../../../nsd/id
     |  |  |           +--rw sap-cpd-id?   -> deref(../nsd-id)/../sapd/id
     |  |  +--rw nfpd* [id]
     |  |     +--rw id                  string
     |  |     +--rw rule?               string
     |  |     +--rw position-desc-id* [id]
     |  |        +--rw id               string
     |  |        +--rw cp-profile-id* [id]
     |  |           +--rw id                              string
     |  |           +--rw constituent-profile-elements* [id]
     |  |              +--rw id        string
     |  |              +--rw cpd-id?   string
     |  +--rw autoscale-rule*                string
     |  +--rw lifecycle-management-script* [event]
     |  |  +--rw event     string
     |  |  +--rw script?   string
     |  +--rw df* [id]
     |  |  +--rw id                                 string
     |  |  +--rw flavour-key?                       -> ../monitored-info/id
     |  |  +--rw vnf-profile* [id]
     |  |  |  +--rw id                                      string
     |  |  |  +--rw vnfd-id                                 -> ../../../../vnfd/id
     |  |  |  +--rw flavour-id                              -> deref(../vnfd-id)/../df/id
     |  |  |  +--rw instantiation-level                     -> deref(../flavour-id)/../instantiation-level/id
     |  |  |  +--rw min-number-of-instances?                uint16
     |  |  |  +--rw max-number-of-instances?                uint16
     |  |  |  +--rw local-affinity-or-anti-affinity-rule* [affinity-type affinity-scope]
     |  |  |  |  +--rw affinity-type     affinity-type
     |  |  |  |  +--rw affinity-scope    affinity-scope
     |  |  |  +--rw affinity-or-anti-affinity-group* [id]
     |  |  |  |  +--rw id    -> ../../../affinity-or-anti-affinity-group/id
     |  |  |  +--rw virtual-link-connectivity* [virtual-link-profile-id]
     |  |  |     +--rw virtual-link-profile-id    -> ../../../virtual-link-profile/id
     |  |  |     +--rw constituent-cpd-id* [constituent-base-element-id]
     |  |  |        +--rw constituent-base-element-id    -> ../../../id
     |  |  |        +--rw constituent-cpd-id?            -> deref(../../../vnfd-id)/../ext-cpd/id
     |  |  +--rw pnf-profile* [id]
     |  |  |  +--rw id                           string
     |  |  |  +--rw pnfd-id?                     -> ../../../../pnfd/id
     |  |  |  +--rw virtual-link-connectivity* [virtual-link-profile-id]
     |  |  |     +--rw virtual-link-profile-id    -> ../../../virtual-link-profile/id
     |  |  |     +--rw constituent-cpd-id* [constituent-base-element-id]
     |  |  |        +--rw constituent-base-element-id    -> ../../../id
     |  |  |        +--rw constituent-cpd-id?            -> deref(../../../pnfd-id)/../ext-cpd/id
     |  |  +--rw virtual-link-profile* [id]
     |  |  |  +--rw id                                      string
     |  |  |  +--rw virtual-link-desc-id                    -> ../../../virtual-link-desc/id
     |  |  |  +--rw flavour-id                              -> deref(../virtual-link-desc-id)/../df/id
     |  |  |  +--rw local-affinity-or-anti-affinity-rule* [affinity-type affinity-scope]
     |  |  |  |  +--rw affinity-type     affinity-type
     |  |  |  |  +--rw affinity-scope    affinity-scope
     |  |  |  +--rw affinity-or-anti-affinity-group* [id]
     |  |  |  |  +--rw id    -> ../../../affinity-or-anti-affinity-group/id
     |  |  |  +--rw max-bitrate-requirements
     |  |  |  |  +--rw root    uint64
     |  |  |  |  +--rw leaf?   uint64
     |  |  |  +--rw min-bitrate-requirements
     |  |  |     +--rw root    uint64
     |  |  |     +--rw leaf?   uint64
     |  |  +--rw scaling-aspect* [id]
     |  |  |  +--rw id               string
     |  |  |  +--rw name?            string
     |  |  |  +--rw description?     string
     |  |  |  +--rw scaling-level?   -> ../../ns-instantiation-level/id
     |  |  +--rw affinity-or-anti-affinity-group* [id]
     |  |  |  +--rw id       string
     |  |  |  +--rw type     affinity-type
     |  |  |  +--rw scope    affinity-scope
     |  |  +--rw ns-instantiation-level* [id]
     |  |  |  +--rw id                               string
     |  |  |  +--rw description?                     string
     |  |  |  +--rw vnf-to-level-mapping* [vnf-profile-id]
     |  |  |  |  +--rw vnf-profile-id         -> ../../../vnf-profile/id
     |  |  |  |  +--rw number-of-instances?   uint32
     |  |  |  +--rw virtual-link-to-level-mapping* [virtual-link-profile-id]
     |  |  |  |  +--rw virtual-link-profile-id    -> ../../../virtual-link-profile/id
     |  |  |  |  +--rw root                       uint64
     |  |  |  |  +--rw leaf?                      uint64
     |  |  |  +--rw ns-to-level-mapping* [ns-profile-id]
     |  |  |     +--rw ns-profile-id          -> ../../../ns-profile/id
     |  |  |     +--rw number-of-instances?   uint32
     |  |  +--rw default-instantiation-level?       -> ../ns-instantiation-level/id
     |  |  +--rw ns-profile* [id]
     |  |  |  +--rw id                                    string
     |  |  |  +--rw nsd-id                                -> ../../../../nsd/id
     |  |  |  +--rw ns-df-id                              -> deref(../nsd-id)/../df/id
     |  |  |  +--rw instantiation-level-id                -> deref(../ns-df-id)/../ns-instantiation-level/id
     |  |  |  +--rw min-number-of-instances?              uint16
     |  |  |  +--rw max-number-of-instances?              uint16
     |  |  |  +--rw affinity-or-anti-affinity-group-id* [id]
     |  |  |  |  +--rw id    -> ../../../affinity-or-anti-affinity-group/id
     |  |  |  +--rw virtual-link-connectivity* [virtual-link-profile-id]
     |  |  |     +--rw virtual-link-profile-id    -> ../../../virtual-link-profile/id
     |  |  |     +--rw constituent-cpd-id* [constituent-base-element-id]
     |  |  |        +--rw constituent-base-element-id    -> ../../../id
     |  |  |        +--rw constituent-cpd-id?            -> deref(../../../nsd-id)/../sapd/id
     |  |  +--rw dependencies* [id]
     |  |  |  +--rw id                             string
     |  |  |  +--rw (primary-id)?
     |  |  |  |  +--:(primary-vnf-profile)
     |  |  |  |  |  +--rw primary-vnf-profile?     -> ../../vnf-profile/vnfd-id
     |  |  |  |  +--:(primary-ns-profile)
     |  |  |  |     +--rw primary-ns-profile?      -> ../../ns-profile/nsd-id
     |  |  |  +--rw (secondary-id)?
     |  |  |     +--:(secondary-vnf-profile)
     |  |  |     |  +--rw secondary-vnf-profile?   -> ../../vnf-profile/vnfd-id
     |  |  |     +--:(secondary-ns-profile)
     |  |  |        +--rw secondary-ns-profile?    -> ../../ns-profile/nsd-id
     |  |  +--rw monitored-info* [id]
     |  |     +--rw id                      string
     |  |     +--rw vnf-indicator-info
     |  |     |  +--rw vnfd-id?         -> ../../../../../vnfd/id
     |  |     |  +--rw vnf-indicator    -> deref(../vnfd-id)/../indicator/id
     |  |     +--rw monitoring-parameter
     |  |        +--rw id?                   string
     |  |        +--rw name?                 string
     |  |        +--rw performance-metric    string
     |  |        +--rw collection-period?    string
     |  +--rw signature?                     string
     |  +--rw algorithm?                     string
     |  +--rw certificate?                   string
     +--rw pnfd* [id]
        +--rw id                            string
        +--rw function-description?         string
        +--rw provider?                     string
        +--rw version                       string
        +--rw invariant-id?                 string
        +--rw name?                         string
        +--rw ext-cpd* [id]
        |  +--rw id                string
        |  +--rw layer-protocol*   identityref
        |  +--rw role?             identityref
        |  +--rw description?      string
        |  +--rw protocol* [associated-layer-protocol]
        |  |  +--rw associated-layer-protocol    identityref
        |  |  +--rw address-data* [type]
        |  |     +--rw type               identityref
        |  |     +--rw l2-address-data
        |  |     |  +--rw mac-address-assignment    boolean
        |  |     +--rw l3-address-data
        |  |        +--rw ip-address-assignment     boolean
        |  |        +--rw floating-ip-activated     boolean
        |  |        +--rw ip-address-type?          enumeration
        |  |        +--rw number-of-ip-addresses?   uint32
        |  +--rw trunk-mode?       boolean
        +--rw security* [signature]
        |  +--rw signature      string
        |  +--rw algorithm?     string
        |  +--rw certificate?   string
        +--rw geographical-location-info?   string
        +--rw security-group-rule* [id]
           +--rw id                string
           +--rw description?      string
           +--rw direction?        enumeration
           +--rw ether-type?       enumeration
           +--rw protocol?         enumeration
           +--rw port-range-min?   uint16
           +--rw port-range-max?   uint16
```

### NST tree

**Navigable version:**

**<http://osm-download.etsi.org/repository/osm/debian/ReleaseNINE/docs/osm-im/osm_im_trees/nst.html>**

**Plain text version:**

```text
module: nst
  +--rw nst* [id]
     +--rw id                           string
     +--rw name                         string
     +--rw SNSSAI-identifier
     |  +--rw slice-service-type      network-slice-type
     |  +--rw slice-differentiator?   string
     +--rw quality-of-service
     |  +--rw id                        uint16
     |  +--rw resource-type?            resource-type
     |  +--rw priority-level?           uint16
     |  +--rw packet-delay-budget?      uint16
     |  +--rw packet-error-rate?        uint16
     |  +--rw default-max-data-burst?   uint16
     +--rw netslice-subnet* [id]
     |  +--rw id                          string
     |  +--rw description?                string
     |  +--rw is-shared-nss?              boolean
     |  +--rw nsd-ref                     -> /nsd:nsd-catalog/nsd/id
     |  +--rw instantiation-parameters
     |     +--rw vimAccountId    string
     |     +--rw ssh_keys?       string
     |     +--rw vnf* [member-vnf-index]
     |     |  +--rw member-vnf-index    string
     |     |  +--rw vimAccountId?       string
     |     |  +--rw vdu* [id]
     |     |  |  +--rw id           string
     |     |  |  +--rw volume* [name]
     |     |  |  |  +--rw name             string
     |     |  |  |  +--rw vim-volume-id    string
     |     |  |  +--rw interface* [name]
     |     |  |     +--rw name                    string
     |     |  |     +--rw ip-address?             inet:ip-address
     |     |  |     +--rw mac-address?            ietf-yang:mac-address
     |     |  |     +--rw floating-ip-required?   boolean
     |     |  +--rw internal-vld* [name]
     |     |     +--rw name                         string
     |     |     +--rw vim-network-name?            string
     |     |     +--rw ip-profile
     |     |     |  +--rw ip-version?        inet:ip-version
     |     |     |  +--rw subnet-address?    inet:ip-prefix
     |     |     |  +--rw gateway-address?   inet:ip-address
     |     |     |  +--rw dns-server* [address]
     |     |     |  |  +--rw address    inet:ip-address
     |     |     |  +--rw dhcp-params
     |     |     |     +--rw enabled?         boolean
     |     |     |     +--rw count?           uint8
     |     |     |     +--rw start-address?   inet:ip-address
     |     |     +--rw internal-connection-point* [id-ref]
     |     |        +--rw id-ref        string
     |     |        +--rw ip-address?   inet:ip-address
     |     +--rw vld* [name]
     |        +--rw name                         string
     |        +--rw vim-network-name?            string
     |        +--rw ip-profile
     |        |  +--rw ip-version?        inet:ip-version
     |        |  +--rw subnet-address?    inet:ip-prefix
     |        |  +--rw gateway-address?   inet:ip-address
     |        |  +--rw dns-server* [address]
     |        |  |  +--rw address    inet:ip-address
     |        |  +--rw dhcp-params
     |        |     +--rw enabled?         boolean
     |        |     +--rw count?           uint8
     |        |     +--rw start-address?   inet:ip-address
     |        +--rw vnfd-connection-point-ref* [member-vnf-index-ref vnfd-connection-point-ref]
     |           +--rw member-vnf-index-ref         -> /nsd:nsd-catalog/nsd/constituent-vnfd/member-vnf-index
     |           +--rw vnfd-connection-point-ref    -> /vnfd:vnfd-catalog/vnfd/connection-point/name
     |           +--rw ip-address?                  inet:ip-address
     +--rw netslice-connection-point* [name]
     |  +--rw name                              string
     |  +--rw floating-ip-required?             boolean
     |  +--rw (connection)?
     |     +--:(netslice-vld-ref)
     |     |  +--rw netslice-vld-id-ref?        -> ../../netslice-vld/id
     |     +--:(nsd-connection-point-ref)
     |        +--rw nsd-id-ref?                 -> /nsd:nsd-catalog/nsd/id
     |        +--rw nsd-connection-point-ref?   -> /nsd:nsd-catalog/nsd/connection-point/name
     +--rw netslice-vld* [id]
     |  +--rw id                          string
     |  +--rw name?                       string
     |  +--rw short-name?                 string
     |  +--rw vendor?                     string
     |  +--rw description?                string
     |  +--rw version?                    string
     |  +--rw type?                       manotypes:virtual-link-type
     |  +--rw root-bandwidth?             uint64
     |  +--rw leaf-bandwidth?             uint64
     |  +--rw provider-network
     |  |  +--rw physical-network?   string
     |  |  +--rw segmentation_id?    uint32
     |  +--rw mgmt-network?               boolean
     |  +--rw nss-connection-point-ref* [nss-ref nsd-connection-point-ref]
     |     +--rw nss-ref                     -> /nst/netslice-subnet/id
     |     +--rw nsd-connection-point-ref    -> /nsd:nsd-catalog/nsd/connection-point/name
     |     +--rw ip-address?                 inet:ip-address
     +--rw netslicefgd* [id]
        +--rw id             string
        +--rw name?          string
        +--rw short-name?    string
        +--rw vendor?        string
        +--rw description?   string
        +--rw version?       string
        +--rw rsp* [id]
        |  +--rw id                          string
        |  +--rw name?                       string
        |  +--rw nsd-connection-point-ref* [nsd-ref]
        |     +--rw nsd-ref                     -> /nsd:nsd-catalog/nsd/id
        |     +--rw order?                      uint8
        |     +--rw nsd-connection-point-ref?   -> /nsd:nsd-catalog/nsd/connection-point/name
        +--rw classifier* [id]
           +--rw id                          string
           +--rw name?                       string
           +--rw rsp-id-ref?                 -> ../../rsp/id
           +--rw match-attributes* [id]
           |  +--rw id                        string
           |  +--rw ip-proto?                 uint8
           |  +--rw source-ip-address?        inet:ip-address
           |  +--rw destination-ip-address?   inet:ip-address
           |  +--rw source-port?              inet:port-number
           |  +--rw destination-port?         inet:port-number
           +--rw nsd-ref?                    -> /nsd:nsd-catalog/nsd/id
           +--rw nsd-connection-point-ref?   -> /nsd:nsd-catalog/nsd/connection-point/name
```

## OSM URN Namespace

### Introduction

OSM is maintaining a namespace registry as a sub tree of the ETSI root `urn:etsi` namespace. A description of the top level ETSI URN namespace can be found on the [ETSI PNNS page](https://portal.etsi.org/PNNS/GenericAllocation/ETSIURNNamespace.aspx)

The OSM URN namespace has the following structure:

```text
urn:etsi:osm:<OSM sub-tree>
```

The following sections describe the basic structure of the sub-tree and how a entry in the registry can be requested.

### OSM sub-tree

| OSM sub-tree |    Full Namespace    |  Type  |                         Description                          |
| :----------: | :------------------: | :----: | :----------------------------------------------------------: |
|     yang     | `urn:etsi:osm:yang`  | string | Namespace for YANG modules specified in OSM.[OSM/IM.git](https://osm.etsi.org/gitweb/?p=osm/IM.git;a=tree;f=models/yang) |
|    yang:augments    | `urn:etsi:osm:yang:augments`  | string | Namespace for YANG modules specified in OSM augmenting SOL006.[OSM/IM.git](https://osm.etsi.org/gitweb/?p=osm/IM.git;a=tree;f=models/yang/augments) |

#### Type definitions

```python
string = (ALPHA)0*(ALPHANUMERIC/-/_)
## a string starts with an upper- or lower-case letter followed by zero or more upper- or lower-case letters, integer numbers, hyphens or underscores

ALPHA = {a-zA-Z}
ALPHANUMERIC = {a-zA-Z0-9}
```

### Requesting an entry

The procedure to request an entry is described as follows.

- **Step #1:** Send an e-mail to the <OSM_TSC@list.etsi.org> mailing list indicating the name of the sub-tree, the contact person and the purpose of that subtree.
- **Step #2:** The TSC will review the proposal.
  - If the application is accepted, a new entry in the "OSM URN Namespace" will be enabled.
  - If the application cannot be accepted, it will be communicated to the contact person together with any additional information to be considered for resolving the "non-acceptance".

## Release EIGHT DESCRIPTOR Documentation (OLD)

The link below contains a detailed description of the OSM Release EIGHT Information Model, previous to the alignment to ETSI NFV SOL006. It can still be used as a reference to compare the previous IM and the current IM.

[OSM Release EIGHT Information Model](http://osm-download.etsi.org/repository/osm/debian/ReleaseEIGHT/docs/osm-im/osm_im_trees/)

## Release TWO Descriptor Documentation (OLD)

The document below contains a detailed description of the OSM Release TWO Models. Although a bit outdated, it can still be used as a reference to understand some of the fields in the VNFD and NSD.

[OSM Release TWO Information Model](https://osm.etsi.org/wikipub/images/2/26/OSM_R2_Information_Model.pdf)
